/**
 * @fileOverview
 * @name network.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @typedef {Object} NetworkController~InterfaceParams - Configuration parameters for a network interface.
 * @property {String} bootproto - [|none] The protocol to be used at boot time.
 * @property {String} dns1 - Name server IP address - option one.
 * @property {String} dns2 - Name server IP address - option two.
 * @property {String} gateway - Gateway device IP address.
 * @property {String} ipaddr - IP address for the interface.
 * @property {String} mode - [static|disabled|dhcp] Operation mode for the interface.
 * @property {String} name - Interface name.
 * @property {String} netmask - Network mask of this network, in dot-decimal notation.
 * @property {String} onboot - [yes|] Enable/Disable activation of interface at boot time.
 */

/**
 * @class NetworkController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {boolean[]} valid_ipaddr - Information list about the validity of the IP address field, for each network interface.
 * @property {boolean[]} valid_Netmask - Information list about the validity of the netmask address field, for each network interface.
 * @property {boolean[]} valid_gateway - Information list about the validity of the Gateway address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS1 address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {boolean[]} loading - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {NetworkController~InterfaceParams[]} interfaces - List of network parameters.
 */

angular.module('ag561app').controller('NetworkController', ['$http', '$log', '$location',
                                   function($http, $log, $location) {
  var url_network = '/api/config/network';
  var ctrl = this;
  ctrl.loading = false;
  ctrl.invalid_config = false;
  ctrl.invalid_ipaddr = false;
  ctrl.invalid_netmask = false;
  ctrl.invalid_gateway = false;
  ctrl.invalid_dns1 = false;
  ctrl.invalid_dns2 = false;
  ctrl.show_ip = true;
  ctrl.show_switch = false;
  ctrl.show_cpu = false;

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof NetworkController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
  };

  /**
   * Get network config and DNS parameters.
   * @function get
   * @memberof NetworkController
   */
  ctrl.get = function() {
    $http({
        method: 'GET',
        url: url_network
        }).then(function(response) {
            // First function handles success
            ctrl.ip = response.data.ip;
            ctrl.switch = response.data.switch;

            // Check non mandatory parameters
            if (!ctrl.switch['monitor_port'])
              ctrl.switch.monitor_port = 'none';

            // Set some variables
            ctrl.switch.port1.invalid_allowed_vids = false;
            ctrl.switch.port2.invalid_allowed_vids = false;

        }, function(response) {
            // Second function handles error
            ctrl.error = response.data;
        });
    }

  /**
   * Save the new configuration.
   * @function save
   * @memberof NetworkController
   */
  ctrl.save = function() {

    // Remove variables that cannot go to API
    delete ctrl.switch.port1.invalid_allowed_vids;
    delete ctrl.switch.port2.invalid_allowed_vids;
    delete ctrl.switch.cpu.invalid_allowed_vids;
    
    var data = {
      'switch'    : ctrl.switch,
      'ip'        : ctrl.ip
    }

    ctrl.loading = true;
    // PUT data to API
    $http({
      method: 'PUT',
      url: url_network,
      data: data
      }).then(function(response) {
          // First function handles success
          ctrl.loading = false;
          ctrl.alert_success = true;
          ctrl.get();
      }, function(response) {
          // Second function handles error
          ctrl.error = response.data;
          ctrl.alert_danger = true;
      });
  };

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save' config and show the 'danger message'.
   *
   * @function __check_invalid_config
   * @memberof NetworkController
   */
  __check_invalid_config = function() {
    if (ctrl.invalid_ipaddr || ctrl.invalid_netmask || ctrl.invalid_gateway ||
        ctrl.invalid_dns1 || ctrl.invalid_dns2 ||
        ctrl.invalid_pppoe_username || ctrl.invalid_pppoe_password ||
        ctrl.invalid_ntp || ctrl.invalid_syslog ||
        ctrl.invalid_allowed_vids) {
      ctrl.invalid_config = true;
      ctrl.alert_danger = true;
    } else {
      ctrl.invalid_config = false;
      ctrl.alert_danger = false;
    }
  };
  
  var ipformat = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$");

  ctrl.validate_ip_config = function() {
      /* Check static IP address and netmask */
      ctrl.invalid_ipaddr = ctrl.ip.type == 'static' && ctrl.ip.addr && !ctrl.ip.addr.match(ipformat);
      ctrl.invalid_netmask = ctrl.ip.type == 'static' && ctrl.ip.mask && !ctrl.ip.mask.match(ipformat);
      
      /* Default Gateway is optional. Only check if it is not empty */
      ctrl.invalid_gateway = ctrl.ip.gateway && !ctrl.ip.gateway.match(ipformat);
      
      /* DNS Servers are optional. Only check if it is not empty */
      ctrl.invalid_dns1 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[0] && !ctrl.ip.dns[0].match(ipformat);
      ctrl.invalid_dns2 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[1] && !ctrl.ip.dns[1].match(ipformat);
      
      /* Validate PPPoE username and password */
      ctrl.invalid_pppoe_username = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_username.match(/^([\S]+)$/);
      ctrl.invalid_pppoe_password = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_password.match(/^([\S]+)$/);

      __check_invalid_config();
  };

  /* Server eg.: aligera:1234 or 192.168.2.1:1234 or 10.10.10.1 */
  var server_port_regex = /^([\w.-]+)(:[\d]+)?$/;
      
  
  ctrl.validateNTP = function() {
    ctrl.invalid_ntp = ctrl.ip.ntp && !ctrl.ip.ntp.match(server_port_regex);
     __check_invalid_config();
  };

  ctrl.validateSyslog = function() {
    ctrl.invalid_syslog = ctrl.ip.syslog && !ctrl.ip.syslog.match(server_port_regex);
    __check_invalid_config();
  };

  /**
   * Check if port allowed VLAN IDs are valid. If not valid, sets a control variable called
   * invalid_allowed_vids, for the port passed as parameter. Also, calls __check_invalid_config()
   * function to disable the save button.
   *
   * @function validate_allowed_vids
   * @memberof NetworkController
   */
  ctrl.validate_allowed_vids = function(port) {
    port.invalid_allowed_vids = false;

    // Split array by spaces or commas
    var vids = port.allowed_vids.split(/[ ,]+/);
    for (var i = 0; i < vids.length; i++) {
      vid = parseInt(vids[i], 10);
      // Check VID range and if it doesnt have letter in it
      if (vid < 2 || vid > 4094 || vids[i].search(/[a-zA-Z]+/) >= 0)
        port.invalid_allowed_vids = true;
    }

    if (port.invalid_allowed_vids == true)
      ctrl.invalid_allowed_vids = true;
    else
      ctrl.invalid_allowed_vids = false;

    __check_invalid_config();
  };

  ctrl.get();

}]);
